/*********************************************
 *  Implementação da Classe Controle Serial  *
 *  HexAvoider                               *
 *  Jose Lu                                  *
 *********************************************/
 
#include "serial.h"
 
/*
 * Implementação da Classe Serial
 */  

//Constructor da Classe Serial
Serial::Serial(){

	//Configuração (8N1 : 8 bits, sem bit de paridade, 1 stopbit)
    new_tio.c_cflag &= ~PARENB;       //desabilita bit de paridade
    new_tio.c_cflag &= ~CSTOPB;       //1 stop bit
    new_tio.c_cflag &= ~CSIZE;        //máscara para bits de dados
    new_tio.c_cflag |= CS8;           //8 bit de dados
    
    cfsetispeed(&new_tio, BAUD_RATE); //Configura taxa de transmissão de entrada
 	cfsetospeed(&new_tio, BAUD_RATE); //Configura taxa de transmissão de saída

	fd=-1;
}

//Estabelece conexão com a porta serial com as configurações pré-estabelecidas
int Serial::open_port(){
	
	fd = open(PORTA_SERIAL,O_RDWR | O_NOCTTY | O_NONBLOCK);   //Tenta estabelecer conexão com a porta serial, 
	                                                          //retornando um descritor para o arquivo (fd)	
	if (fd == -1) {   // Não conseguiu estabelecer conexão com a porta serial
     fprintf(stderr, "open_port: Não conseguiu abrir a porta %s - %s\n",
             PORTA_SERIAL,strerror(errno));

   }else {           //Conseguiu estabelecer conexão com a porta serial
		fcntl (fd, F_SETOWN, getpid());    //configura o processo para interrupção
		fcntl (fd, F_SETFL, FASYNC);       //configura para método assíncrono
		tcgetattr(fd, &old_tio);           //guarda as configurações atuais da porta no <old_tio>
		tcflush(fd, TCIFLUSH);             //limpa os dados da porta
		tcsetattr(fd, TCSANOW, &new_tio);  //seta as novas configurações na porta através da descritor do arquivo (fd) do <new_tio>
	}
	return fd;
}

//fecha a porta serial
int Serial::close_port(){
	if ( close(fd) < 0 )   //fecha a porta serial
	   fprintf(stderr, "close_port: Não conseguiu fechar a porta %s - %s\n",
             PORTA_SERIAL,strerror(errno));
	return fd;
}

//escreve o conteúdo do buffer na porta serial
int  Serial::write_port(char *buffer){

	if (write(fd, buffer, strlen(buffer)) < 0)   //escreve buffer na porta
	   fprintf(stderr, "write_port: Não conseguiu escrever na porta %s - %s\n",
             PORTA_SERIAL,strerror(errno));	
	return fd;
} 

//lê os dados da porta serial
int  Serial::read_port(char *buffer){
	
	int pos; //quantidade de caracteres lido da porta serial

	if ((pos = read(fd,&buffer,BUFFER_TAM)) > 0)  //retorna conteúdo da porta em <buffer>
		buffer[pos]='\0';
	else{   //caso falha de leitura ou conteúdo nulo
		buffer[0]='\0';
/*	   fprintf(stderr, "read_port: Erro ao ler conteúdo da porta %s - %s\n",
             PORTA_SERIAL,strerror(errno));		*/
	}
	return fd;
}  		

//Destructor da Classe
Serial::~Serial(){
	
	tcsetattr(fd, TCSANOW, &old_tio);  //restaura os parâmetros da porta serial
	close_port();                      //fecha a porta serial
}
