/*
 HEXAVD1.C
 HexAvoider Source Program.
*/

#include <io51.h>

typedef unsigned char byte;
typedef unsigned int  word;

#define high_byte(cnt_16bit) (unsigned char)(cnt_16bit>>8)
#define low_byte(cnt_16bit) (unsigned char)(cnt_16bit)

// OUTPUT PORT
#define SERVO1_CON   P1.5     // LEFT R/C SERVO MOTOR
#define SERVO2_CON   P1.6     // RIGHT R/C SERVO MOTOR
#define SERVO3_CON   P1.7     // CENTER R/C SERVO MOTOR

#define LED_LEFT     P1.3
#define LED_RIGHT    P1.4
#define LED_ANTENNA  T0

#define TRX_SEL      T1       // RF MODELE MODE

#define IRLED_LEFT   P1.0
#define IRLED_RIGHT  P1.1

#define LED_D6       P0.0
#define LED_D7       P0.1
#define LED_D8       P0.2
#define LED_D9       P0.3
#define BUZZER       P0.4

// INPUT PORT
#define IR_READ      P1.2
#define L_ANTENNA    INT0
#define R_ANTENNA    INT1
#define DIPSW_HIGH   P2.1
#define DIPSW_LOW    P2.0

// CONSTANT DEFINITION
#define ON           0
#define OFF          1
#define YES          1
#define NO           0
#define PULSE_LOW    0
#define PULSE_HIGH   1
#define SPEED_LOW    220              // Low Speed Delay
#define SPEED_HIGH   170              // High Speed Delay
#define INIT_800US   0x10000-822+6    // 892 us

#define C_LEG_CENTER   112            // Center Value of Center Legs
#define L_LEG_CENTER   112            // Center Value of Left Legs
#define R_LEG_CENTER   112            // Center Value of Right Legs

#define C_LEG_LIMIT    50             // Center: Left & Right Movement Range
#define L_LEG_LIMIT    60             // Left: Front & Rear Movement Range
#define R_LEG_LIMIT    60             // Right: Front & Rear Movement Range

// GLOBAL VARIABLES
bit  sense_update_flag;
byte pulse1, pulse2, pulse3;
byte task_index, mode;
byte sensor_count;
byte sense_result, limit_sense_result;
word movement_delay;
word max16bit, temp;

void delay( unsigned int i )
{
     data unsigned int j;
     for( ; i>0; i--) for( j=91 ; j>0; j--);
}

void delay2( unsigned int i )
{
     for( ; i>0; i--);
}

void delay_100us_interrupt( void )
{
     data unsigned char i;
     for(i=14 ; i>0; i--);
}

void center_left_leg_up(void)
{
    pulse3 = C_LEG_CENTER-C_LEG_LIMIT;
}

void center_right_leg_up(void)
{
    pulse3 = C_LEG_CENTER+C_LEG_LIMIT;
}

void left_leg_front_right_leg_back(void)
{
    pulse1 = L_LEG_CENTER+L_LEG_LIMIT;
    pulse2 = R_LEG_CENTER+R_LEG_LIMIT;
}

void left_leg_back_right_leg_front(void)
{
    pulse1 = L_LEG_CENTER-L_LEG_LIMIT;
    pulse2 = R_LEG_CENTER-R_LEG_LIMIT;
}

void left_right_legs_front(void)
{
    pulse1 = L_LEG_CENTER+L_LEG_LIMIT;
    pulse2 = R_LEG_CENTER-R_LEG_LIMIT;
}

void left_right_legs_back(void)
{
    pulse1 = L_LEG_CENTER-L_LEG_LIMIT;
    pulse2 = R_LEG_CENTER+R_LEG_LIMIT;
}

void move_forward(void)
{
    if(sense_result) return;
    center_left_leg_up();
    delay(movement_delay);
    if(sense_result) return;

    left_leg_back_right_leg_front();
    delay(movement_delay);
    if(sense_result) return;

    center_right_leg_up();
    delay(movement_delay);
    if(sense_result) return;

    left_leg_front_right_leg_back();
    delay(movement_delay);
    if(sense_result) return;

    return ;
}

void move_backward(void)
{
    center_left_leg_up();
    delay(movement_delay);

    left_leg_front_right_leg_back();
    delay(movement_delay);
    delay(movement_delay);

    center_right_leg_up();
    delay(movement_delay);

    left_leg_back_right_leg_front();
    delay(movement_delay);
    delay(movement_delay);
}

void turn_left(void)
{
    center_left_leg_up();
    delay(movement_delay);

    left_right_legs_front();
    delay(movement_delay);
    delay(movement_delay);

    center_right_leg_up();
    delay(movement_delay);

    left_right_legs_back();
    delay(movement_delay);
    delay(movement_delay);
}

void turn_right(void)
{
    center_left_leg_up();
    delay(movement_delay);

    left_right_legs_back();
    delay(movement_delay);
    delay(movement_delay);

    center_right_leg_up();
    delay(movement_delay);

    left_right_legs_front();
    delay(movement_delay);
    delay(movement_delay);
}

void beep1(void)
{
    unsigned char i;
    for(i=0; i<120; i++)
    {
        BUZZER=1;
        delay2(30);
        BUZZER=0;
        delay2(30);
    }
}

void beep2(void)
{

    unsigned char i;
    for(i=0; i<120; i++)
    {
        BUZZER=OFF;
        delay2(30);
        BUZZER=ON;
        delay2(30);
    }

    for(i=0; i<50; i++)
    {
        BUZZER=OFF;
        delay2(60);
        BUZZER=ON;
        delay2(60);
    }

}

interrupt [0x1B] using[3] void T1_int (void)
{
    SERVO1_CON = SERVO2_CON = SERVO3_CON = PULSE_LOW;
    ET1 = TR1 = 0;
}

interrupt [0x0B] using[2] void T0_int (void)
{
    TH0=high_byte(63236);          // HIGH OF (65536-2300)
    TL0=low_byte(63236);           // LOW OF (65536-2300)
                                   // 2300 x 1.085 us = 2.5 ms

    switch(++task_index)
    {
        case 1 : if(pulse1)
                 {
                   max16bit=INIT_800US;
                   temp=pulse1;
                   temp=(temp<<2)+pulse1;
                   max16bit-=temp;
                   TH1=high_byte(max16bit);
                   TL1=low_byte(max16bit);
                   SERVO1_CON=PULSE_HIGH;
                   ET1=TR1=1;
                 }
                 break;
        case 2 : if(pulse2)
                 {
                   max16bit=INIT_800US;
                   temp=pulse2;
                   temp=(temp<<2)+pulse2;
                   max16bit-=temp;
                   TH1=high_byte(max16bit);
                   TL1=low_byte(max16bit);
                   SERVO2_CON=PULSE_HIGH;
                   ET1=TR1=1;
                 }
                 break;
        case 3 : if(pulse3)
                 {
                   max16bit=INIT_800US;
                   temp=pulse3;
                   temp=(temp<<2)+pulse3;
                   max16bit-=temp;
                   TH1=high_byte(max16bit);
                   TL1=low_byte(max16bit);
                   SERVO3_CON=PULSE_HIGH;
                   ET1=TR1=1;
                 }
                 break;
        case 4 : if(++sensor_count==5)
                 {
                   IRLED_LEFT=ON;
                   delay_100us_interrupt();
                   LED_LEFT=IR_READ;
                   IRLED_LEFT=OFF;
                 }
                 break;
        case 5 : if(sensor_count==10)
                 {
                   IRLED_RIGHT=ON;
                   delay_100us_interrupt();
                   LED_RIGHT=IR_READ;
                   IRLED_RIGHT=OFF;
                 }
                 break;
        case 6 : if(sensor_count==15)
                 {
                   sense_result=0;
                   if(!L_ANTENNA || !LED_LEFT ) sense_result++;
                   if(!R_ANTENNA || !LED_RIGHT) sense_result+=2;
                   if(!L_ANTENNA || !R_ANTENNA) LED_ANTENNA=ON;
                   else LED_ANTENNA=OFF;
                   sensor_count=0;
                   sense_update_flag=YES;
                 }
                 break;
        case 7 : break;
        case 8 : task_index=0;
                 break;
        default : task_index=0;
    }
}

void sensor_test()
{
    beep1(); delay(300); beep1(); delay(300); beep1();

    while(1)
    {
        if(sense_result && sense_update_flag==YES)
        {
            beep1();
            sense_update_flag=NO;
        }
    }
}

void servo_positioning()
{
    beep1(); delay(300); beep1(); delay(300); beep1(); delay(300); beep1();
    pulse3 = C_LEG_CENTER-C_LEG_LIMIT; delay(1000);
    pulse3 = C_LEG_CENTER+C_LEG_LIMIT; delay(1000);
    pulse3 = C_LEG_CENTER; delay(1000);

    pulse1 = L_LEG_CENTER+L_LEG_LIMIT; delay(1000);
    pulse1 = L_LEG_CENTER-L_LEG_LIMIT; delay(1000);
    pulse1 = L_LEG_CENTER; delay(1000);

    pulse2 = R_LEG_CENTER-R_LEG_LIMIT; delay(1000);
    pulse2 = R_LEG_CENTER+R_LEG_LIMIT; delay(1000);
    pulse2 = R_LEG_CENTER; delay(1000);

    while(1);
}

void initialize(void)
{
    IRLED_LEFT = IRLED_RIGHT = OFF;            // IRLED OFF
    SERVO1_CON = SERVO2_CON = SERVO3_CON = PULSE_LOW;
                                              // RESET SERVO CONTROL BITS
    pulse1=pulse2=pulse3=0;                   // DISABLE SERVO MOTORS
    movement_delay = SPEED_LOW;
    sense_update_flag = NO;

    beep2();

    LED_LEFT = ON ; LED_RIGHT =OFF;
    LED_D6 = ON   ;
    delay(300);

    LED_LEFT = OFF; LED_RIGHT = ON;
    LED_D6 = OFF  ; LED_D7 = ON;
    delay(300);

    LED_LEFT = ON ; LED_RIGHT = OFF;
    LED_D7 = OFF  ; LED_D8 = ON;
    delay(300);

    LED_LEFT = OFF; LED_RIGHT = ON;
    LED_D8 = OFF  ; LED_D9 = ON;
    delay(300);

    LED_LEFT = LED_D9 = OFF;
    P2=0xFF;
}

void init_timer( void )
{
    ET0=0;
    ET1=0;
    TMOD=0x11;           // 16 BIT TIMER MODE
    TH1=TH1=TH0=TL0=0x00;
}

/*
void my_test(void)   // YOU CAN TEST YOUR OWN ROUTINE.
{                    // IF YOU WANT TO DO, CHANGE case 2:(in switch in main())
}
*/

void main(void)
{
    initialize();

    init_timer();

    IE=0x80;          // ENABLE ALL INTERRUPT
    ET0=1;            // ENABLE TIMER0 INTERRUPT
    TR0=1;            // RUN TIMER0

    switch(~P2 & 0x03)
    {
        case 0: LED_D9=ON; movement_delay = SPEED_HIGH; break;
        case 1: LED_D8=0; break;
        case 2: LED_D7=0; sensor_test(); break;
//        case 2: LED_D7=0; my_test(); break;
        case 3: LED_D6=0; servo_positioning(); break;
    }

    pulse3 = C_LEG_CENTER;
    pulse1 = L_LEG_CENTER;
    pulse2 = R_LEG_CENTER; delay(1000);
    
    while(1)
    {
        move_forward();
        switch(sense_result)
        {
            case 1: beep1(); turn_right(); break;
            case 2: beep1(); turn_left(); break;
            case 3: beep1(); move_backward(); turn_right(); break;
        }
    };
}
